@@ -36,6 +36,7 @@ import io.netty.util.internal.PlatformDependent;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.EnumSet;
 import java.util.List;
 import java.util.PrimitiveIterator.OfLong;
 import java.util.concurrent.Executors;
@@ -60,8 +61,10 @@ import org.apache.bookkeeper.common.util.MathUtils;
 import org.apache.bookkeeper.common.util.Watcher;
 import org.apache.bookkeeper.conf.ServerConfiguration;
 import org.apache.bookkeeper.meta.LedgerManager;
+import org.apache.bookkeeper.stats.Gauge;
 import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.stats.StatsLogger;
+import org.apache.bookkeeper.stats.annotations.StatsDoc;
 import org.apache.bookkeeper.util.DiskChecker;
 import org.apache.commons.lang3.StringUtils;
 
@@ -84,19 +87,42 @@ public class DbLedgerStorage implements LedgerStorage {
             / MB;
     private static final long DEFAULT_READ_CACHE_MAX_SIZE_MB = (long) (0.25 * PlatformDependent.maxDirectMemory())
             / MB;
+
+    static final String READ_AHEAD_CACHE_BATCH_SIZE = "dbStorage_readAheadCacheBatchSize";
+    private static final int DEFAULT_READ_AHEAD_CACHE_BATCH_SIZE = 100;
+
+    // use the storage assigned to ledger 0 for flags.
+    // if the storage configuration changes, the flags may be lost
+    // but in that case data integrity should kick off anyhow.
+    private static final long STORAGE_FLAGS_KEY = 0L;
     private int numberOfDirs;
     private List<SingleDirectoryDbLedgerStorage> ledgerStorageList;
 
     // Keep 1 single Bookie GC thread so the the compactions from multiple individual directories are serialized
     private ScheduledExecutorService gcExecutor;
-    private DbLedgerStorageStats stats;
 
     protected ByteBufAllocator allocator;
 
+    // parent DbLedgerStorage stats (not per directory)
+    private static final String MAX_READAHEAD_BATCH_SIZE = "readahead-max-batch-size";
+    private static final String MAX_WRITE_CACHE_SIZE = "write-cache-max-size";
+
+    @StatsDoc(
+            name = MAX_READAHEAD_BATCH_SIZE,
+            help = "the configured readahead batch size"
+    )
+    private Gauge<Integer> readaheadBatchSizeGauge;
+
+    @StatsDoc(
+            name = MAX_WRITE_CACHE_SIZE,
+            help = "the configured write cache size"
+    )
+    private Gauge<Long> writeCacheSizeGauge;
+
     @Override
     public void initialize(ServerConfiguration conf, LedgerManager ledgerManager, LedgerDirsManager ledgerDirsManager,
-            LedgerDirsManager indexDirsManager, StateManager stateManager, CheckpointSource checkpointSource,
-            Checkpointer checkpointer, StatsLogger statsLogger, ByteBufAllocator allocator) throws IOException {
+                           LedgerDirsManager indexDirsManager, StatsLogger statsLogger, ByteBufAllocator allocator)
+            throws IOException {
         long writeCacheMaxSize = getLongVariableOrDefault(conf, WRITE_CACHE_MAX_SIZE_MB,
                 DEFAULT_WRITE_CACHE_MAX_SIZE_MB) * MB;
         long readCacheMaxSize = getLongVariableOrDefault(conf, READ_AHEAD_CACHE_MAX_SIZE_MB,
@@ -116,6 +142,7 @@ public class DbLedgerStorage implements LedgerStorage {
 
         long perDirectoryWriteCacheSize = writeCacheMaxSize / numberOfDirs;
         long perDirectoryReadCacheSize = readCacheMaxSize / numberOfDirs;
+        int readAheadCacheBatchSize = conf.getInt(READ_AHEAD_CACHE_BATCH_SIZE, DEFAULT_READ_AHEAD_CACHE_BATCH_SIZE);
 
         gcExecutor = Executors.newSingleThreadScheduledExecutor(new DefaultThreadFactory("GarbageCollector"));
 
@@ -127,29 +154,61 @@ public class DbLedgerStorage implements LedgerStorage {
             dirs[0] = ledgerDir.getParentFile();
             LedgerDirsManager ldm = new LedgerDirsManager(conf, dirs, ledgerDirsManager.getDiskChecker(), statsLogger);
             ledgerStorageList.add(newSingleDirectoryDbLedgerStorage(conf, ledgerManager, ldm, indexDirsManager,
-                    stateManager, checkpointSource, checkpointer, statsLogger, gcExecutor, perDirectoryWriteCacheSize,
-                    perDirectoryReadCacheSize));
+                    statsLogger, gcExecutor, perDirectoryWriteCacheSize,
+                    perDirectoryReadCacheSize, readAheadCacheBatchSize));
             ldm.getListeners().forEach(ledgerDirsManager::addLedgerDirsListener);
         }
 
-        this.stats = new DbLedgerStorageStats(
-            statsLogger,
-            () -> ledgerStorageList.stream().mapToLong(SingleDirectoryDbLedgerStorage::getWriteCacheSize).sum(),
-            () -> ledgerStorageList.stream().mapToLong(SingleDirectoryDbLedgerStorage::getWriteCacheCount).sum(),
-            () -> ledgerStorageList.stream().mapToLong(SingleDirectoryDbLedgerStorage::getReadCacheSize).sum(),
-            () -> ledgerStorageList.stream().mapToLong(SingleDirectoryDbLedgerStorage::getReadCacheCount).sum()
-        );
+        // parent DbLedgerStorage stats (not per directory)
+        readaheadBatchSizeGauge = new Gauge<Integer>() {
+            @Override
+            public Integer getDefaultValue() {
+                return readAheadCacheBatchSize;
+            }
+
+            @Override
+            public Integer getSample() {
+                return readAheadCacheBatchSize;
+            }
+        };
+        statsLogger.registerGauge(MAX_READAHEAD_BATCH_SIZE, readaheadBatchSizeGauge);
+
+        writeCacheSizeGauge = new Gauge<Long>() {
+            @Override
+            public Long getDefaultValue() {
+                return perDirectoryWriteCacheSize;
+            }
+
+            @Override
+            public Long getSample() {
+                return perDirectoryWriteCacheSize;
+            }
+        };
+        statsLogger.registerGauge(MAX_WRITE_CACHE_SIZE, writeCacheSizeGauge);
     }
 
     @VisibleForTesting
     protected SingleDirectoryDbLedgerStorage newSingleDirectoryDbLedgerStorage(ServerConfiguration conf,
             LedgerManager ledgerManager, LedgerDirsManager ledgerDirsManager, LedgerDirsManager indexDirsManager,
-            StateManager stateManager, CheckpointSource checkpointSource, Checkpointer checkpointer,
-            StatsLogger statsLogger, ScheduledExecutorService gcExecutor, long writeCacheSize, long readCacheSize)
+            StatsLogger statsLogger, ScheduledExecutorService gcExecutor, long writeCacheSize, long readCacheSize,
+            int readAheadCacheBatchSize)
             throws IOException {
         return new SingleDirectoryDbLedgerStorage(conf, ledgerManager, ledgerDirsManager, indexDirsManager,
-                stateManager, checkpointSource, checkpointer, statsLogger, allocator, gcExecutor, writeCacheSize,
-                readCacheSize);
+                                                  statsLogger, allocator, gcExecutor, writeCacheSize, readCacheSize,
+                                                  readAheadCacheBatchSize);
+    }
+
+    @Override
+    public void setStateManager(StateManager stateManager) {
+        ledgerStorageList.forEach(s -> s.setStateManager(stateManager));
+    }
+    @Override
+    public void setCheckpointSource(CheckpointSource checkpointSource) {
+        ledgerStorageList.forEach(s -> s.setCheckpointSource(checkpointSource));
+    }
+    @Override
+    public void setCheckpointer(Checkpointer checkpointer) {
+        ledgerStorageList.forEach(s -> s.setCheckpointer(checkpointer));
     }
 
     @Override
@@ -166,56 +225,61 @@ public class DbLedgerStorage implements LedgerStorage {
 
     @Override
     public boolean ledgerExists(long ledgerId) throws IOException {
-        return getLedgerSorage(ledgerId).ledgerExists(ledgerId);
+        return getLedgerStorage(ledgerId).ledgerExists(ledgerId);
+    }
+
+    @Override
+    public boolean entryExists(long ledgerId, long entryId) throws IOException, BookieException {
+        return getLedgerStorage(ledgerId).entryExists(ledgerId, entryId);
     }
 
     @Override
     public boolean setFenced(long ledgerId) throws IOException {
-        return getLedgerSorage(ledgerId).setFenced(ledgerId);
+        return getLedgerStorage(ledgerId).setFenced(ledgerId);
     }
 
     @Override
-    public boolean isFenced(long ledgerId) throws IOException {
-        return getLedgerSorage(ledgerId).isFenced(ledgerId);
+    public boolean isFenced(long ledgerId) throws IOException, BookieException {
+        return getLedgerStorage(ledgerId).isFenced(ledgerId);
     }
 
     @Override
     public void setMasterKey(long ledgerId, byte[] masterKey) throws IOException {
-        getLedgerSorage(ledgerId).setMasterKey(ledgerId, masterKey);
+        getLedgerStorage(ledgerId).setMasterKey(ledgerId, masterKey);
     }
 
     @Override
     public byte[] readMasterKey(long ledgerId) throws IOException, BookieException {
-        return getLedgerSorage(ledgerId).readMasterKey(ledgerId);
+        return getLedgerStorage(ledgerId).readMasterKey(ledgerId);
     }
 
     @Override
     public long addEntry(ByteBuf entry) throws IOException, BookieException {
         long ledgerId = entry.getLong(entry.readerIndex());
-        return getLedgerSorage(ledgerId).addEntry(entry);
+        return getLedgerStorage(ledgerId).addEntry(entry);
     }
 
     @Override
-    public ByteBuf getEntry(long ledgerId, long entryId) throws IOException {
-        return getLedgerSorage(ledgerId).getEntry(ledgerId, entryId);
+    public ByteBuf getEntry(long ledgerId, long entryId) throws IOException, BookieException {
+        return getLedgerStorage(ledgerId).getEntry(ledgerId, entryId);
     }
 
     @Override
-    public long getLastAddConfirmed(long ledgerId) throws IOException {
-        return getLedgerSorage(ledgerId).getLastAddConfirmed(ledgerId);
+    public long getLastAddConfirmed(long ledgerId) throws IOException, BookieException {
+        return getLedgerStorage(ledgerId).getLastAddConfirmed(ledgerId);
     }
 
     @Override
     public boolean waitForLastAddConfirmedUpdate(long ledgerId, long previousLAC,
             Watcher<LastAddConfirmedUpdateNotification> watcher) throws IOException {
-        return getLedgerSorage(ledgerId).waitForLastAddConfirmedUpdate(ledgerId, previousLAC, watcher);
+        return getLedgerStorage(ledgerId).waitForLastAddConfirmedUpdate(ledgerId, previousLAC, watcher);
     }
 
     @Override
     public void cancelWaitForLastAddConfirmedUpdate(long ledgerId,
                                                     Watcher<LastAddConfirmedUpdateNotification> watcher)
             throws IOException {
-        getLedgerSorage(ledgerId).cancelWaitForLastAddConfirmedUpdate(ledgerId, watcher);
+        getLedgerStorage(ledgerId).cancelWaitForLastAddConfirmedUpdate(ledgerId, watcher);
     }
 
     @Override
@@ -234,7 +298,7 @@ public class DbLedgerStorage implements LedgerStorage {
 
     @Override
     public void deleteLedger(long ledgerId) throws IOException {
-        getLedgerSorage(ledgerId).deleteLedger(ledgerId);
+        getLedgerStorage(ledgerId).deleteLedger(ledgerId);
     }
 
     @Override
@@ -244,28 +308,28 @@ public class DbLedgerStorage implements LedgerStorage {
 
     @Override
     public void setExplicitLac(long ledgerId, ByteBuf lac) throws IOException {
-        getLedgerSorage(ledgerId).setExplicitLac(ledgerId, lac);
+        getLedgerStorage(ledgerId).setExplicitLac(ledgerId, lac);
     }
 
     @Override
-    public ByteBuf getExplicitLac(long ledgerId) throws IOException {
-        return getLedgerSorage(ledgerId).getExplicitLac(ledgerId);
+    public ByteBuf getExplicitLac(long ledgerId) throws IOException, BookieException {
+        return getLedgerStorage(ledgerId).getExplicitLac(ledgerId);
     }
 
     public long addLedgerToIndex(long ledgerId, boolean isFenced, byte[] masterKey,
                                  LedgerCache.PageEntriesIterable pages) throws Exception {
-        return getLedgerSorage(ledgerId).addLedgerToIndex(ledgerId, isFenced, masterKey, pages);
+        return getLedgerStorage(ledgerId).addLedgerToIndex(ledgerId, isFenced, masterKey, pages);
     }
 
     public long getLastEntryInLedger(long ledgerId) throws IOException {
-        return getLedgerSorage(ledgerId).getEntryLocationIndex().getLastEntryInLedger(ledgerId);
+        return getLedgerStorage(ledgerId).getEntryLocationIndex().getLastEntryInLedger(ledgerId);
     }
 
     public long getLocation(long ledgerId, long entryId) throws IOException {
-        return getLedgerSorage(ledgerId).getEntryLocationIndex().getLocation(ledgerId, entryId);
+        return getLedgerStorage(ledgerId).getEntryLocationIndex().getLocation(ledgerId, entryId);
     }
 
-    private SingleDirectoryDbLedgerStorage getLedgerSorage(long ledgerId) {
+    private SingleDirectoryDbLedgerStorage getLedgerStorage(long ledgerId) {
         return ledgerStorageList.get(MathUtils.signSafeMod(ledgerId, numberOfDirs));
     }
 
@@ -278,8 +342,8 @@ public class DbLedgerStorage implements LedgerStorage {
         return Iterables.concat(listIt);
     }
 
-    public ByteBuf getLastEntry(long ledgerId) throws IOException {
-        return getLedgerSorage(ledgerId).getLastEntry(ledgerId);
+    public ByteBuf getLastEntry(long ledgerId) throws IOException, BookieException {
+        return getLedgerStorage(ledgerId).getLastEntry(ledgerId);
     }
 
     @VisibleForTesting
@@ -315,7 +379,7 @@ public class DbLedgerStorage implements LedgerStorage {
 
         EntryLocationIndex entryLocationIndex = new EntryLocationIndex(serverConf,
                 (basePath, subPath, dbConfigType, conf1) ->
-                        new KeyValueStorageRocksDB(basePath, subPath, DbConfigType.Small, conf1, true),
+                        new KeyValueStorageRocksDB(basePath, subPath, DbConfigType.Default, conf1, true),
                 ledgerBasePath, NullStatsLogger.INSTANCE);
         try {
             long lastEntryId = entryLocationIndex.getLastEntryInLedger(ledgerId);
@@ -339,6 +403,11 @@ public class DbLedgerStorage implements LedgerStorage {
         ledgerStorageList.stream().forEach(SingleDirectoryDbLedgerStorage::forceGC);
     }
 
+    @Override
+    public void forceGC(Boolean forceMajor, Boolean forceMinor) {
+        ledgerStorageList.stream().forEach(s -> s.forceGC(forceMajor, forceMinor));
+    }
+
     @Override
     public boolean isInForceGC() {
         return ledgerStorageList.stream().anyMatch(SingleDirectoryDbLedgerStorage::isInForceGC);
@@ -369,4 +438,34 @@ public class DbLedgerStorage implements LedgerStorage {
         throw new UnsupportedOperationException(
                 "getListOfEntriesOfLedger method is currently unsupported for DbLedgerStorage");
     }
+
+    @Override
+    public void setLimboState(long ledgerId) throws IOException {
+        getLedgerStorage(ledgerId).setLimboState(ledgerId);
+    }
+
+    @Override
+    public boolean hasLimboState(long ledgerId) throws IOException {
+        return getLedgerStorage(ledgerId).hasLimboState(ledgerId);
+    }
+
+    @Override
+    public void clearLimboState(long ledgerId) throws IOException {
+        getLedgerStorage(ledgerId).clearLimboState(ledgerId);
+    }
+
+    @Override
+    public EnumSet<StorageState> getStorageStateFlags() throws IOException {
+        return getLedgerStorage(STORAGE_FLAGS_KEY).getStorageStateFlags();
+    }
+
+    @Override
+    public void setStorageStateFlag(StorageState flag) throws IOException {
+        getLedgerStorage(STORAGE_FLAGS_KEY).setStorageStateFlag(flag);
+    }
+
+    @Override
+    public void clearStorageStateFlag(StorageState flag) throws IOException {
+        getLedgerStorage(STORAGE_FLAGS_KEY).clearStorageStateFlag(flag);
+    }
 }
