@@ -57,9 +57,9 @@ public class DefaultBookieAddressResolver implements BookieAddressResolver {
         } catch (BKException.BKBookieHandleNotAvailableException ex) {
             if (BookieSocketAddress.isDummyBookieIdForHostname(bookieId)) {
                 log.debug("Resolving dummy bookie Id {} using legacy bookie resolver", bookieId);
-                return BookieSocketAddress.resolveDummyBookieId(bookieId);
+                return BookieSocketAddress.resolveLegacyBookieId(bookieId);
             }
-            log.info("Cannot resolve {}, bookie is unknown", bookieId, ex.toString());
+            log.info("Cannot resolve {}, bookie is unknown {}", bookieId, ex.toString());
             throw new BookieIdNotResolvedException(bookieId, ex);
         } catch (Exception ex) {
             if (ex instanceof InterruptedException) {
