@@ -47,7 +47,6 @@ import org.apache.bookkeeper.net.BookieId;
 import org.apache.bookkeeper.proto.BookkeeperInternalCallbacks.GenericCallback;
 import org.apache.bookkeeper.stats.OpStatsLogger;
 import org.apache.bookkeeper.util.MathUtils;
-import org.apache.bookkeeper.util.SafeRunnable;
 import org.apache.bookkeeper.versioning.Versioned;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -69,6 +68,7 @@ class LedgerCreateOp {
     final int writeQuorumSize;
     final int ackQuorumSize;
     final Map<String, byte[]> customMetadata;
+    final int metadataFormatVersion;
     final byte[] passwd;
     final BookKeeper bk;
     final DigestType digestType;
@@ -108,6 +108,7 @@ class LedgerCreateOp {
             EnumSet<WriteFlag> writeFlags,
             BookKeeperClientStats clientStats) {
         this.bk = bk;
+        this.metadataFormatVersion = bk.getConf().getLedgerMetadataFormatVersion();
         this.ensembleSize = ensembleSize;
         this.writeQuorumSize = writeQuorumSize;
         this.ackQuorumSize = ackQuorumSize;
@@ -175,6 +176,7 @@ class LedgerCreateOp {
         if (customMetadata != null) {
             metadataBuilder.withCustomMetadata(customMetadata);
         }
+        metadataBuilder.withMetadataFormatVersion(metadataFormatVersion);
         if (bk.getConf().getStoreSystemtimeAsLedgerCreationTime()) {
             metadataBuilder.withCreationTime(System.currentTimeMillis()).storingCreationTime(true);
         }
@@ -270,16 +272,7 @@ class LedgerCreateOp {
         } else {
             createOpLogger.registerSuccessfulEvent(MathUtils.elapsedNanos(startTime), TimeUnit.NANOSECONDS);
         }
-        if (lh != null) { // lh is null in case of errors
-            lh.executeOrdered(new SafeRunnable() {
-                @Override
-                public void safeRun() {
-                    cb.createComplete(rc, lh, ctx);
-                }
-            });
-        } else {
-            cb.createComplete(rc, null, ctx);
-        }
+        cb.createComplete(rc, lh, ctx);
     }
 
     public static class CreateBuilderImpl implements CreateBuilder {
