@@ -90,7 +90,6 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
     protected boolean enforceMinNumRacksPerWriteQuorum;
     protected boolean ignoreLocalNodeInPlacementPolicy;
 
-    public static final String REPP_DNS_RESOLVER_CLASS = "reppDnsResolverClass";
     public static final String REPP_RANDOM_READ_REORDERING = "ensembleRandomReadReordering";
 
     static final int RACKNAME_DISTANCE_FROM_LEAVES = 1;
@@ -270,8 +269,8 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
                 }
             } catch (RuntimeException re) {
                 if (!conf.getEnforceMinNumRacksPerWriteQuorum()) {
-                    LOG.warn("Failed to initialize DNS Resolver {}, used default subnet resolver because {}",
-                            dnsResolverName, re.getMessage());
+                    LOG.warn("Failed to initialize DNS Resolver {}, used default subnet resolver ",
+                            dnsResolverName, re);
                     dnsResolver = new DefaultResolver(this::getDefaultRack);
                     dnsResolver.setBookieAddressResolver(bookieAddressResolver);
                 } else {
@@ -414,9 +413,9 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
                 }
                 return PlacementResult.of(addrs, PlacementPolicyAdherence.FAIL);
             }
-            //Choose different rack nodes.
-            String curRack = null;
+
             for (int i = 0; i < ensembleSize; i++) {
+                String curRack;
                 if (null == prevNode) {
                     if ((null == localNode) || defaultRack.equals(localNode.getNetworkLocation())) {
                         curRack = NodeBase.ROOT;
@@ -424,24 +423,11 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
                         curRack = localNode.getNetworkLocation();
                     }
                 } else {
-                    if (!curRack.startsWith("~")) {
-                        curRack = "~" + prevNode.getNetworkLocation();
-                    } else {
-                        curRack = curRack + NetworkTopologyImpl.NODE_SEPARATOR + prevNode.getNetworkLocation();
-                    }
+                    curRack = "~" + prevNode.getNetworkLocation();
                 }
                 boolean firstBookieInTheEnsemble = (null == prevNode);
-                try {
-                    prevNode = selectRandomFromRack(curRack, excludeNodes, ensemble, ensemble);
-                } catch (BKNotEnoughBookiesException e) {
-                    if (!curRack.equals(NodeBase.ROOT)) {
-                        curRack = NodeBase.ROOT;
-                        prevNode = selectFromNetworkLocation(curRack, excludeNodes, ensemble, ensemble,
-                                !enforceMinNumRacksPerWriteQuorum || firstBookieInTheEnsemble);
-                    } else {
-                        throw e;
-                    }
-                }
+                prevNode = selectFromNetworkLocation(curRack, excludeNodes, ensemble, ensemble,
+                        !enforceMinNumRacksPerWriteQuorum || firstBookieInTheEnsemble);
             }
             List<BookieId> bookieList = ensemble.toList();
             if (ensembleSize != bookieList.size()) {
@@ -532,10 +518,9 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
                         networkLoc, excludeBookies);
                 throw e;
             }
-            LOG.warn("Failed to choose a bookie from network location {}, "
-                    + "the bookies in the network location are {}, excluded bookies {}, "
-                    + "current ensemble {}, fallback to choose bookie randomly from the cluster.",
-                     networkLoc, topology.getLeaves(networkLoc), excludeBookies, ensemble);
+            LOG.warn("Failed to choose a bookie from {} : "
+                     + "excluded {}, fallback to choose bookie randomly from the cluster.",
+                     networkLoc, excludeBookies);
             // randomly choose one from whole cluster, ignore the provided predicate.
             return selectRandom(1, excludeBookies, predicate, ensemble).get(0);
         }
@@ -558,10 +543,6 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
              * the whole cluster and exclude the racks specified at
              * <tt>excludeRacks</tt>.
              */
-            LOG.warn("Failed to choose a bookie node from network location {}, "
-                    + "the bookies in the network location are {}, excluded bookies {}, "
-                    + "current ensemble {}, fallback to choose bookie randomly from the cluster.",
-                networkLoc, topology.getLeaves(networkLoc), excludeBookies, ensemble);
             return selectFromNetworkLocation(excludeRacks, excludeBookies, predicate, ensemble, fallbackToRandom);
         }
     }
@@ -1036,6 +1017,9 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
     @Override
     public PlacementPolicyAdherence isEnsembleAdheringToPlacementPolicy(List<BookieId> ensembleList,
             int writeQuorumSize, int ackQuorumSize) {
+        if (CollectionUtils.isEmpty(ensembleList)) {
+            return PlacementPolicyAdherence.FAIL;
+        }
         int ensembleSize = ensembleList.size();
         int minNumRacksPerWriteQuorumForThisEnsemble = Math.min(writeQuorumSize, minNumRacksPerWriteQuorum);
         HashSet<String> racksInQuorum = new HashSet<String>();
@@ -1045,7 +1029,11 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
             for (int j = 0; j < writeQuorumSize; j++) {
                 bookie = ensembleList.get((i + j) % ensembleSize);
                 try {
-                    racksInQuorum.add(knownBookies.get(bookie).getNetworkLocation());
+                    if (knownBookies.containsKey(bookie)) {
+                        racksInQuorum.add(knownBookies.get(bookie).getNetworkLocation());
+                    } else {
+                        LOG.debug("bookie {} is not in the list of knownBookies", bookie);
+                    }
                 } catch (Exception e) {
                     /*
                      * any issue/exception in analyzing whether ensemble is
@@ -1074,7 +1062,11 @@ public class RackawareEnsemblePlacementPolicyImpl extends TopologyAwareEnsembleP
         readLock.lock();
         try {
             for (BookieId bookie : ackedBookies) {
-                rackCounter.add(knownBookies.get(bookie).getNetworkLocation());
+                if (knownBookies.containsKey(bookie)) {
+                    rackCounter.add(knownBookies.get(bookie).getNetworkLocation());
+                } else {
+                    LOG.debug("bookie {} is not in the list of knownBookies", bookie);
+                }
             }
 
             // Check to make sure that ensemble is writing to `minNumberOfRacks`'s number of racks at least.
