@@ -36,6 +36,7 @@ import java.util.concurrent.Future;
 
 import lombok.Cleanup;
 
+import org.apache.bookkeeper.bookie.BookieResources;
 import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.client.ClientUtil;
 import org.apache.bookkeeper.client.LedgerHandle;
@@ -50,12 +51,13 @@ import org.apache.bookkeeper.http.service.HttpServiceResponse;
 import org.apache.bookkeeper.meta.LedgerManager;
 import org.apache.bookkeeper.meta.LedgerManagerFactory;
 import org.apache.bookkeeper.meta.LedgerUnderreplicationManager;
+import org.apache.bookkeeper.meta.MetadataBookieDriver;
 import org.apache.bookkeeper.net.BookieSocketAddress;
 import org.apache.bookkeeper.replication.AuditorElector;
 import org.apache.bookkeeper.server.http.service.BookieInfoService;
 import org.apache.bookkeeper.server.http.service.BookieStateReadOnlyService.ReadOnlyState;
 import org.apache.bookkeeper.server.http.service.BookieStateService.BookieState;
-import org.apache.bookkeeper.server.http.service.ClusterInfoService;
+import org.apache.bookkeeper.stats.NullStatsLogger;
 import org.apache.bookkeeper.test.BookKeeperClusterTestCase;
 import org.junit.Before;
 import org.junit.Test;
@@ -75,7 +77,7 @@ public class TestHttpService extends BookKeeperClusterTestCase {
     public TestHttpService() {
         super(numberOfBookies);
         try {
-            File tmpDir = createTempDir("bookie_http", "test");
+            File tmpDir = tmpDirs.createNew("bookie_http", "test");
             baseConf.setJournalDirName(tmpDir.getPath())
               .setLedgerDirNames(
                 new String[]{tmpDir.getPath()});
@@ -90,9 +92,14 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         super.setUp();
         baseConf.setMetadataServiceUri(zkUtil.getMetadataServiceUri());
         baseClientConf.setStoreSystemtimeAsLedgerCreationTime(true);
+
+        MetadataBookieDriver metadataDriver = BookieResources.createMetadataDriver(
+                baseConf, NullStatsLogger.INSTANCE);
+
         this.bkHttpServiceProvider = new BKHttpServiceProvider.Builder()
-            .setBookieServer(bs.get(numberOfBookies - 1))
+            .setBookieServer(serverByIndex(numberOfBookies - 1))
             .setServerConfiguration(baseConf)
+            .setLedgerManagerFactory(metadataDriver.getLedgerManagerFactory())
             .build();
     }
 
@@ -335,7 +342,7 @@ public class TestHttpService extends BookKeeperClusterTestCase {
 
         //3,  delete first ledger, should return OK, and should only get 3 ledgers after delete.
         HashMap<String, String> params = Maps.newHashMap();
-        Long ledgerId = Long.valueOf(lh[0].getId());
+        Long ledgerId = lh[0].getId();
         params.put("ledger_id", ledgerId.toString());
         HttpServiceRequest request3 = new HttpServiceRequest(null, HttpServer.Method.DELETE, params);
         HttpServiceResponse response3 = deleteLedgerService.handle(request3);
@@ -384,7 +391,7 @@ public class TestHttpService extends BookKeeperClusterTestCase {
 
         //2,  parameters for GET first ledger, should return OK, and contains metadata
         HashMap<String, String> params = Maps.newHashMap();
-        Long ledgerId = Long.valueOf(lh[0].getId());
+        Long ledgerId = lh[0].getId();
         params.put("ledger_id", ledgerId.toString());
         HttpServiceRequest request2 = new HttpServiceRequest(null, HttpServer.Method.GET, params);
         HttpServiceResponse response2 = getLedgerMetaService.handle(request2);
@@ -433,7 +440,7 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         //2,  parameters for GET first ledger, should return OK
         // no start/end entry id, so return all the 100 entries.
         HashMap<String, String> params = Maps.newHashMap();
-        Long ledgerId = Long.valueOf(lh[0].getId());
+        Long ledgerId = lh[0].getId();
         params.put("ledger_id", ledgerId.toString());
         HttpServiceRequest request2 = new HttpServiceRequest(null, HttpServer.Method.GET, params);
         HttpServiceResponse response2 = readLedgerEntryService.handle(request2);
@@ -596,7 +603,7 @@ public class TestHttpService extends BookKeeperClusterTestCase {
 
     AuditorElector auditorElector;
     private Future<?> startAuditorElector() throws Exception {
-        String addr = bs.get(0).getBookieId().toString();
+        String addr = addressByIndex(0).toString();
         ServerConfiguration conf = TestBKConfiguration.newServerConfiguration();
         conf.setAuditorPeriodicBookieCheckInterval(1);
         conf.setMetadataServiceUri("zk://" + zkUtil.getZooKeeperConnectString() + "/ledgers");
@@ -857,6 +864,7 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         HttpServiceRequest request1 = new HttpServiceRequest(null, HttpServer.Method.GET, null);
         HttpServiceResponse response1 = bookieStateServer.handle(request1);
         assertEquals(HttpServer.StatusCode.OK.getValue(), response1.getStatusCode());
+        assertEquals("OK", response1.getBody());
 
         // Try using POST instead of GET
         HttpServiceRequest request2 = new HttpServiceRequest(null, HttpServer.Method.POST, null);
@@ -864,7 +872,9 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         assertEquals(HttpServer.StatusCode.NOT_FOUND.getValue(), response2.getStatusCode());
 
         // Simulate bookies shutting down
-        bs.forEach(bookieServer -> bookieServer.getBookie().getStateManager().forceToShuttingDown());
+        for (int i = 0; i < bookieCount(); i++) {
+            serverByIndex(i).getBookie().getStateManager().forceToShuttingDown();
+        }
         HttpServiceRequest request3 = new HttpServiceRequest(null, HttpServer.Method.GET, null);
         HttpServiceResponse response3 = bookieStateServer.handle(request3);
         assertEquals(HttpServer.StatusCode.SERVICE_UNAVAILABLE.getValue(), response3.getStatusCode());
@@ -890,34 +900,6 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         assertEquals(HttpServer.StatusCode.NOT_FOUND.getValue(), response2.getStatusCode());
     }
 
-    @Test
-    public void testGetClusterInfo() throws Exception {
-        HttpEndpointService clusterInfoServer = bkHttpServiceProvider
-                .provideHttpEndpointService(HttpServer.ApiType.CLUSTER_INFO);
-
-        HttpServiceRequest request1 = new HttpServiceRequest(null, HttpServer.Method.GET, null);
-        HttpServiceResponse response1 = clusterInfoServer.handle(request1);
-        assertEquals(HttpServer.StatusCode.OK.getValue(), response1.getStatusCode());
-        LOG.info("Get response: {}", response1.getBody());
-
-        ClusterInfoService.ClusterInfo info = JsonUtil.fromJson(response1.getBody(),
-                ClusterInfoService.ClusterInfo.class);
-        assertFalse(info.isAuditorElected());
-        assertTrue(info.getAuditorId().length() == 0);
-        assertFalse(info.isClusterUnderReplicated());
-        assertTrue(info.isLedgerReplicationEnabled());
-        assertTrue(info.getTotalBookiesCount() > 0);
-        assertTrue(info.getWritableBookiesCount() > 0);
-        assertTrue(info.getReadonlyBookiesCount() == 0);
-        assertTrue(info.getUnavailableBookiesCount() == 0);
-        assertTrue(info.getTotalBookiesCount() == info.getWritableBookiesCount());
-
-        // Try using POST instead of GET
-        HttpServiceRequest request2 = new HttpServiceRequest(null, HttpServer.Method.POST, null);
-        HttpServiceResponse response2 = clusterInfoServer.handle(request2);
-        assertEquals(HttpServer.StatusCode.NOT_FOUND.getValue(), response2.getStatusCode());
-    }
-
     @Test
     public void testBookieReadOnlyState() throws Exception {
         HttpEndpointService bookieStateServer = bkHttpServiceProvider
@@ -984,13 +966,5 @@ public class TestHttpService extends BookKeeperClusterTestCase {
         response = bookieReadOnlyService.handle(request);
         readOnlyState = JsonUtil.fromJson(response.getBody(), ReadOnlyState.class);
         assertFalse(readOnlyState.isReadOnly());
-
-        //forceReadonly to writable
-        baseConf.setForceReadOnlyBookie(true);
-        baseConf.setReadOnlyModeEnabled(true);
-        restartBookies();
-        request = new HttpServiceRequest(JsonUtil.toJson(new ReadOnlyState(false)), HttpServer.Method.PUT,  null);
-        response = bookieReadOnlyService.handle(request);
-        assertEquals(400, response.getStatusCode());
     }
 }
