@@ -608,7 +608,7 @@ class ReadAheadEntryReader implements
 
         if (cause instanceof EndOfLogSegmentException) {
             // we reach end of the log segment
-            moveToNextLogSegment();
+            moveToNextLogSegment(currentSegmentReader);
             return;
         }
         if (cause instanceof IOException) {
@@ -906,11 +906,15 @@ class ReadAheadEntryReader implements
         return true;
     }
 
-    void moveToNextLogSegment() {
+    void moveToNextLogSegment(final SegmentReader prevSegmentReader) {
         orderedSubmit(new CloseableRunnable() {
             @Override
             public void safeRun() {
-                unsafeMoveToNextLogSegment();
+                // Do not move forward if previous enqueued runnable
+                // already moved the segment forward.
+                if (prevSegmentReader == currentSegmentReader) {
+                    unsafeMoveToNextLogSegment();
+                }
             }
         });
     }
